﻿// *****************************************************************
// *                                                               *
// *       Copyright (c) Fast Corporation, 2010                    *
// *                                                               *
// *   All Rights Reserved.  Unpublished rights  reserved  under   *
// *   the copyright laws of the Japan.                            *
// *                                                               *
// *   The software contained on this media  is  proprietary  to   *
// *   and  embodies  the  confidential  technology  of  Fast      *
// *   Corporation.  Possession, use,  duplication  or             *
// *   dissemination of the software and media is authorized only  *
// *   pursuant to a valid written license from Fast Corporation.  *
// *                                                               *
// *****************************************************************

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

// -----------------------------------------------------------------
// FVIL Sample Program
// メインダイアログ
//
// 作成者 : N.Moriguchi
//
// 履歴 : 2012/05/16 初版
//
// FVC07 Continue Grab Sample Program
// -----------------------------------------------------------------
namespace FVC07_ContGrabSample
{
    public partial class Form_FVC07_CONTGRABSAMPLE : Form
    {
        //連続取り込み用画像メモリ格納領域
        private List<FVIL.Data.CFviImage> m_Images = new List<FVIL.Data.CFviImage>();
        //FVC07クラス
        private FVIL.Video.CFviVideoFVC07 m_video = null;
        //処理時間計測クラス
        private FVIL.CFviTimeCounter m_timer = new FVIL.CFviTimeCounter();

        //iniファイル名（フルパス）
        public string m_strIniFilePath = null;

        //画像オブジェクト番号
        private int m_intBufImgIndex;

        //ビデオデバイスのオープン状態
        private bool m_flgVideoOpened;
        //フリーランの状態
        private bool m_flgFreerun;
        
        public Form_FVC07_CONTGRABSAMPLE()
        {
            InitializeComponent();
        }

        /// <summary>
        /// フォーム表示開始
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 使用許諾を確認します。
        /// </remarks>
        private void Form1_Shown(object sender, EventArgs e)
        {
            dialogLicence licence_dialog = new dialogLicence();
            if (licence_dialog.ShowDialog(this) != DialogResult.OK)
            {
                this.Close();
                //使用許諾が得られない場合はサンプルプログラムの動作を終了
                return;
            }

             //iniファイル選択
            fnSetIniFile();

        }

        /// <summary>
        /// メインフォーム初期化
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// メインフォームの初期化処理。
        /// </remarks>
        private void Form1_Load(object sender, EventArgs e)
        {
            // FVILの初期化
            // アプリケーションを起動した後、 この関数を呼び出してライブラリを初期化してください。 
            //  この時、ライセンスの認証が行われます。試用期間中であっても、この処理は必ず行う必要が有ります。 
            // 初期化を行わなかった場合は、プロテクトされたクラスの実行ができません。
            FVIL._SetUp.InitVisionLibrary();

            //状態変数の初期化
            m_flgVideoOpened = false;
            m_flgFreerun = false;

            //計測時間テキストボックス初期化
            TextBoxTime.Text = "0.000";
            //フレーム数テキストボックス初期化
            TextBoxFrame.Text = "0";
            //キャプチャー数テキストボックス初期化
            TextBoxCaptureNumber.Text = "10";
            //表示画像メモリ番号初期化
            TextBoxBufImgIndex.Text = "0";

            //トリガモードコンボボックス初期化
            ComboBoxTrigger.Items.Clear();
            ComboBoxTrigger.Items.Add("ノーマル");
            ComboBoxTrigger.Items.Add("内部トリガ");
            ComboBoxTrigger.Items.Add("外部トリガ");
            ComboBoxTrigger.Items.Add("外部トリガ2");
            ComboBoxTrigger.SelectedIndex = 0;

        }

        /// <summary>
        /// iniファイル選択関数
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// iniファイルの選択をします。
        /// </remarks>
        private void fnSetIniFile()
        {
            //iniファイル設定画面
            Form_BROWSEIN formbroseini = new Form_BROWSEIN();

            //iniファイルの選択
            if (formbroseini.ShowDialog(this) == DialogResult.OK)
            {
                m_strIniFilePath = formbroseini.strIniFilePath;
            }

            if (m_strIniFilePath == null)
            {
                //iniファイルが未選択の場合はサンプルプログラムの動作を終了
                this.Close();
                return;
            }


            if (m_strIniFilePath.Length > 0)
            {
                try
                {
                    //ビデオデバイスがオープンされていた場合はクローズする
                    if (m_video != null)
                    {
                        if (m_video.IsOpen())
                        {
                            m_video.Close();
                            m_flgVideoOpened = false;
                        }
                    }
                    //指定されたカメラ設定ファイルに該当するビデオ入力クラスのインスタンスを生成し、 Open 関数と LoadIniFile(String) 関数を実行して返します。 
                    //iniファイルを指定して、デバイスオープン
                    m_video = (FVIL.Video.CFviVideoFVC07)FVIL.Video.Function.CreateInstance(m_strIniFilePath);
                    m_flgVideoOpened = true;

                    //VideoConfFileの変更後、OpenDeviceを行うと、トリガーモードの変更が伴います
                    //弊社キャプチャボードはトリガーモードの設定後にセットアップタイムが必要になります｡
                    //本プログラムのセットアップタイムは必要最大値である350msを設定しています。
                    //キャプチャボードによってこの時間は異なります。
                    System.Threading.Thread.Sleep(350);

                    //ステータス表示の更新
                    //トリガモード取得
                    ComboBoxTrigger.SelectedIndex = (int)m_video.Trigger;
                    NumericUpDownExposure.Value = m_video.ExposureTime;
                }
                catch (FVIL.CFviException ex)
                {
                    //エラー処理
                    MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (System.Exception ex)
                {
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else
            {
                //iniファイルが未選択の場合はサンプルプログラムの動作を終了
                this.Close();
                return;
            }

        }

        /// <summary>
        /// Grab Oneボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// カメラ取り込み。
        /// </remarks>
        private void ButtonGrabOne_Click(object sender, EventArgs e)
        {
            double msec = 0;

            try
            {
                //フレーム数テキストボックス初期化
                TextBoxFrame.Text = "0";
                //画像メモリ格納領域クリア
                m_Images.Clear();
                //画像メモリ確保数分メモリ格納領域追加
                m_Images.Add(new FVIL.Data.CFviImage());
                //取込画像を正規化
                m_video.ChangeImageSize(m_Images[0]);
                //画像メモリ格納領域をセット
                m_video.ContinuousGrab.Images = m_Images;
                //画像メモリロック
                m_video.ContinuousGrab.MemoryLock(1);

                //時間測定開始
                m_timer.Start();
                //カメラ取り込み
                m_video.ContinuousGrab.GrabSync(1);
                //時間測定終了
                msec = m_timer.Stop();
                //測定時間表示
                TextBoxTime.Text = msec.ToString("0.000");
                //描画imageのセット
                cFviImageView1.Image = m_Images[(int)m_video.ContinuousGrab.QueryCaptureNo().LastFrameNo];
                //再描画
                cFviImageView1.Refresh();
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Grab Continueボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 連続カメラ取り込み開始・停止。
        /// </remarks>
        private void ButtonGrabContinue_Click(object sender, EventArgs e)
        {
            //ボタン表示変更
            if (m_flgFreerun == true)
            {
                //フリーラン停止処理
                fnStopFreerun();
            }
            else
            {
                try
                {
                    //ステータス変更
                    ButtonGrabContinue.Text = "Stop";
                    ButtonGrabOne.Enabled = false;
                    NumericUpDownLockImgNum.Enabled = false;
                    m_flgFreerun = true;
                    //測定時間表示無効
                    TextBoxTime.Text = "0.000";
                    TextBoxTime.Enabled = false;
                    //画像メモリ格納領域クリア
                    m_Images.Clear();
                    for (int LockNum = 0; LockNum < NumericUpDownLockImgNum.Value; LockNum++){
                        //画像メモリ確保数分メモリ格納領域追加
                        m_Images.Add(new FVIL.Data.CFviImage());
                        //取込画像を正規化
                        m_video.ChangeImageSize(m_Images[LockNum]);
                    }
                    //画像メモリ格納領域をセット
                    m_video.ContinuousGrab.Images = m_Images;
                    //画像メモリロック
                    m_video.ContinuousGrab.MemoryLock((uint)NumericUpDownLockImgNum.Value);
                    timer1.Interval = 50;
                    timer1.Start();
                    //連続カメラ取り込み
                    m_video.ContinuousGrab.GrabASync(uint.Parse(TextBoxCaptureNumber.Text));
                }
                catch (FVIL.CFviException ex)
                {
                    //フリーラン停止処理
                    fnStopFreerun();

                    //エラー処理
                    MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (System.Exception ex)
                {
                    //フリーラン停止処理
                    fnStopFreerun();

                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            Application.DoEvents();
        }

        /// <summary>
        /// タイマーイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// タイマーによる表示更新。
        /// </remarks>
        private void timer1_Tick(object sender, EventArgs e)
        {
            try
            {
                if (m_flgFreerun == true)
                {
                    if ((uint.Parse(TextBoxCaptureNumber.Text) > m_video.ContinuousGrab.QueryCaptureNo().TotalFrameCount) || (int.Parse(TextBoxCaptureNumber.Text) == 0))
                    {
                        timer1.Stop();
                        //フレーム数表示更新
                        TextBoxFrame.Text = m_video.ContinuousGrab.QueryCaptureNo().TotalFrameCount.ToString();
                        //最新フレームをセット
                        cFviImageView1.Image = m_Images[(int)m_video.ContinuousGrab.QueryCaptureNo().LastFrameNo];
                        //再描画
                        cFviImageView1.Refresh();
                        int stat;
                        stat = m_video.ContinuousGrab.GrabStatus;
                        timer1.Start();
                        //時間測定開始
                        m_timer.Start();
                    }
                    else
                    {
                        timer1.Stop();
                        //フレーム数表示更新
                        TextBoxFrame.Text = m_video.ContinuousGrab.QueryCaptureNo().TotalFrameCount.ToString();
                        //最新フレームをセット
                        cFviImageView1.Image = m_Images[(int)m_video.ContinuousGrab.QueryCaptureNo().LastFrameNo];
                        //再描画
                        cFviImageView1.Refresh();
                        //フリーラン停止処理
                        fnStopFreerun();
                    }
                }
            }
            catch (FVIL.CFviException ex)
            {
                //フリーラン停止処理
                fnStopFreerun();

                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //フリーラン停止処理
                fnStopFreerun();

                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// フリーラン停止関数
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// フリーラン停止処理と各種ステータスを変更する。
        /// </remarks>
        private void fnStopFreerun()
        {
            try
            {
                //ステータス変更
                ButtonGrabContinue.Text = "Grab Continue";
                ButtonGrabOne.Enabled = true;
                NumericUpDownLockImgNum.Enabled = true;
                m_flgFreerun = false;
                timer1.Stop();
                timer1.Enabled = false;
                //画像取り込みの停止()
                m_video.ContinuousGrab.StopCapture();
            }
            catch (Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
    }

        /// <summary>
        /// ←ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 表示画像メモリの番号を戻す。
        /// </remarks>
        private void ButtonLeft_Click(object sender, EventArgs e)
        {
            int size;

            m_intBufImgIndex = int.Parse(TextBoxBufImgIndex.Text) - 1;
            size = (int)NumericUpDownLockImgNum.Value;
            if (m_intBufImgIndex < 0)
            {
                m_intBufImgIndex = size - 1;
            }
            if (m_intBufImgIndex < 0)
            {
                m_intBufImgIndex = 0;
            }
            TextBoxBufImgIndex.Text = m_intBufImgIndex.ToString();
            if (m_Images.Count > int.Parse(TextBoxBufImgIndex.Text))
            {
                //表示画像をセット
                cFviImageView1.Image = m_Images[int.Parse(TextBoxBufImgIndex.Text)];
                //再描画
                cFviImageView1.Refresh();
            }
        }

        /// <summary>
        /// →ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 表示画像メモリの番号を進める。
        /// </remarks>
        private void ButtonRight_Click(object sender, EventArgs e)
        {
            int size;

            m_intBufImgIndex = int.Parse(TextBoxBufImgIndex.Text) + 1;
            size = (int)NumericUpDownLockImgNum.Value;
            if (m_intBufImgIndex >= size)
            {
                m_intBufImgIndex = 0;
            }
            TextBoxBufImgIndex.Text = m_intBufImgIndex.ToString();
            if (m_Images.Count > int.Parse(TextBoxBufImgIndex.Text))
            {
                
                //表示画像をセット
                cFviImageView1.Image = m_Images[int.Parse(TextBoxBufImgIndex.Text)];
                //再描画
                cFviImageView1.Refresh();
            }
        }

        /// <summary>
        /// Trigger Modeコンボボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ノーマルモード・内部トリガモード・外部トリガモードの選択。
        /// </remarks>
        private void ComboBoxTrigger_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    m_video.Trigger = (uint)ComboBoxTrigger.SelectedIndex;
                    //弊社キャプチャボードはトリガーモードの設定後にセットアップタイムが必要になります｡
                    //本プログラムのセットアップタイムは必要最大値である350msを設定しています。
                    //キャプチャボードによってこの時間は異なります。
                    System.Threading.Thread.Sleep(350);
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Video Exposure値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// カメラ露光時間の設定。
        /// </remarks>
        private void NumericUpDownExposure_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    m_video.ExposureTime = (uint)NumericUpDownExposure.Value;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Sendボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// シリアル通信でカメラ設定を変更します。
        /// </remarks>
        private void ButtonSendSIO_Click(object sender, EventArgs e)
        {
            try
            {
                List<byte> senddata, revdata;
                Byte[] byterevdata;
                int i;

                //送信データを取得
                senddata = new List<byte>(System.Text.Encoding.ASCII.GetBytes(TextBoxSendData.Text));

                //シリアルポートをオープン
                m_video.Sio.Open();

                //データ送信
                revdata = m_video.Sio.SendData(senddata, 0);

                //受信データをByteに変換
                byterevdata = revdata.ToArray();

                //受信データを表示
                TextBoxRecieveData.Text = "";
                for (i = 0; i < revdata.Count; i++)
                {
                    TextBoxRecieveData.Text = System.Text.Encoding.ASCII.GetString(byterevdata);
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            finally
            {
                m_video.Sio.Close();
            }

        }

        /// <summary>
        /// x1ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 等倍で表示します。
        /// </remarks>
        private void ButtonMag1_Click(object sender, EventArgs e)
        {
            //画像の表示倍率の取得または設定を行います。1.0と指定すると等倍で表示します。
            //倍率値は、0.5とした場合は、幅・高さ共に1/2となり、面積が1/4になります。
            //3.0とした場合は、幅・高さ共に3倍となり、面積が9倍になります。

            //表示倍率の設定
            cFviImageView1.Display.Magnification = 1.0;
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// Fitボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 画像全体がビューのクライアント領域に表示できるように表示倍率を縮小(または拡大)します。
        /// </remarks>
        private void ButtonFit_Click(object sender, EventArgs e)
        {
            //画像全体がビューのクライアント領域に表示できるように表示倍率を縮小(または拡大)します
            //表示倍率を調整
            cFviImageView1.FitImageSize();
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// ＋ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 現在の倍率に0.5加えた倍率で表示します。
        /// </remarks>
        private void ButtonPlus_Click(object sender, EventArgs e)
        {
            double Value = 0;
            //現倍率の取得
            Value = cFviImageView1.Display.Magnification;
            //表示倍率の設定
            Value = Value + 0.5;
            cFviImageView1.Display.Magnification = Value;
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// －ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 現在の倍率から0.5引いた倍率で表示します。ただし、倍率が０以下になる場合は倍率0.1で表示します。
        /// </remarks>
        private void ButtonMinus_Click(object sender, EventArgs e)
        {
            double Value = 0;
            //現倍率の取得
            Value = cFviImageView1.Display.Magnification;
            Value = Value - 0.5;
            if (Value <= 0)
            {
                //０以下を設定するとエラーになるので0.1を設定する
                Value = 0.1;
            }
            //表示倍率の設定
            cFviImageView1.Display.Magnification = Value;
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// Ascopeチェックボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ASCOPEの表示を切替えます。
        /// </remarks>
        private void CheckBoxAscope_CheckedChanged(object sender, EventArgs e)
        {
            //ASCOPEオーバレイの設定
            if (CheckBoxAscope.Checked)
            {
                //ASCOPEの表示を有効にする。
                cFviImageView1.Display.OverlayAscope.Enable = true;
                cFviImageView1.Display.OverlayAscope.Active = true;
            }
            else
            {
                //ASCOPEの表示を無効にする。
                cFviImageView1.Display.OverlayAscope.Enable = false;
                cFviImageView1.Display.OverlayAscope.Active = false;
            }
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// ReLoad Ini Fileボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ビデオ設定用iniファイルの再設定。
        /// </remarks>
        private void ButtonReLoadIni_Click(object sender, EventArgs e)
        {
            try
            {
                //フリーラン中ならば、停止する。
	            if (m_flgFreerun == true)
                {
                    //フリーラン停止処理
                    fnStopFreerun();
	            }
                //画像メモリがロックされていたら解除する。
                if (m_video.ContinuousGrab.IsMemoryLocked())
                {
                    //画像メモリロックの解除
                    m_video.ContinuousGrab.MemoryUnlock();
                }

        	    //iniファイル選択
	            fnSetIniFile();
            }
            catch (Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Save Imageボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 表示画像保存。
        /// </remarks>
        private void ButtonSaveImage_Click(object sender, EventArgs e)
        {
            
            SaveFileDialog ofdlg = new SaveFileDialog();

            ofdlg.Filter = "画像ファイル(*.bmp;*.dib;*.png;*.jpg;*.jpeg;*.tif;*.tiff;*.raw;*.pat)|*.bmp;*.dib;*.png;*.jpg;*.jpeg;*.tif;*.tiff;*.raw;*.pat";
            ofdlg.Filter += "|Bitmap ファイル (*.bmp;*.dib)|*.bmp;*.dib";
            ofdlg.Filter += "|Png ファイル (*.png)|*.png";
            ofdlg.Filter += "|Jpeg ファイル (*.jpg;*.jpeg)|*.jpg;*.jpeg";
            ofdlg.Filter += "|Tiff ファイル (*.tif;*.tiff)|*.tif;*.tiff";
            ofdlg.Filter += "|Raw ファイル (*.raw)|*.raw";
            ofdlg.Filter += "|すべてのファイル (*.*)|*.*";
            ofdlg.OverwritePrompt = true;

            if (ofdlg.ShowDialog(this) == DialogResult.OK)
            {
                try
                {
                    //この関数は、簡易的に画像ファイルの保存を行う為のものです。 
                    //この関数で指定できないパラメータを指定する必要があれば、 各画像ファイルアクセスクラスを直接使用してください。 
                    FVIL.File.Function.SaveImageFile(ofdlg.FileName, cFviImageView1.Image);

                }
                catch (FVIL.CFviException ex)
                {
                    System.String msg = System.String.Format("ライブラリが例外を発行しました。code={0} func={1}", ex.ErrorCode, ex.Function);
                    switch ((ex.ErrorCode))
                    {
                        case FVIL.ErrorCode.NOT_SUPPORTED:
                        case FVIL.ErrorCode.INVALID_OBJECT:
                            msg += "\r\n" + "サポートされていない画像オブジェクトです。";
                            msg += "\r\n" + "Raw 形式で保存してください。";
                            break;
                    }
                    //エラー処理
                    MessageBox.Show(msg, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (Exception ex)
                {
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        /// <summary>
        /// 終了処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// アプリケーション終了処理。
        /// </remarks>
        private void Form_FVC071CHSAMPLE_FormClosing(object sender, FormClosingEventArgs e)
        {
            try
            {
                if (m_flgVideoOpened)
                {
                    //ロックされたメモリを解除します。 
                    if (m_video.ContinuousGrab.IsMemoryLocked())
                    {
                        //取り込みの停止
                        m_video.ContinuousGrab.StopCapture();
                        //画像メモリロックの解除
                        m_video.ContinuousGrab.MemoryUnlock();
                    }
                    m_video.Close();
                    m_flgVideoOpened = false;
                }
            }
            catch (Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// キャプチャー数テキストボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// キャプチャー数のクランプ処理。
        /// </remarks>
        private void TextBoxCaptureNumber_TextChanged(object sender, EventArgs e)
        {
            try
            {
                if (int.Parse(TextBoxCaptureNumber.Text) < 0)
                {
                    TextBoxCaptureNumber.Text = "0";
                }
            }
            catch
            {
                //エラー処理
                //数字以外の入力は10に初期化する。
                TextBoxCaptureNumber.Text = "10";
            }
        }

    }
}